#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.12
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { createRequire } from 'node:module';
import { Command } from 'commander';
import chalk from 'chalk';
import getType from 'js-cool/es/getType';
import getUserToken from '@gitmars/core/lib/api/getUserToken';
import { queue } from '@gitmars/core/lib/queue';
import getIsBranchOrCommitExist from '@gitmars/core/lib/git/getIsBranchOrCommitExist';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getCurrentBranch from '@gitmars/core/lib/git/getCurrentBranch';
import getGitConfig from '@gitmars/core/lib/git/getGitConfig';
import getIsMergedTargetBranch from '@gitmars/core/lib/git/getIsMergedTargetBranch';
import checkGitStatus from '@gitmars/core/lib/git/checkGitStatus';
import fetch from '@gitmars/core/lib/git/fetch';
import { createArgs } from '@gitmars/core/lib/utils/command';
import { spawnSync } from '@gitmars/core/lib/spawn';
import echo from '@gitmars/core/lib/utils/echo';
import getConfig from '@gitmars/core/lib/getConfig';
import adminConfig from '#lib/conf/admin';
import i18n from '#lib/locales/index';

const require = createRequire(import.meta.url);
const { green, red } = chalk;
if (!getIsGitProject()) {
  echo(red(i18n.__("The current directory is not a git project directory")));
  process.exit(1);
}
const { appName } = getGitConfig();
const config = getConfig();
const userInfoApi = config.apis && config.apis.userInfo && config.apis.userInfo.url || config.api;
const mergeRequestModule = require.resolve("@gitmars/core/lib/api/mergeRequest");
const { approve, clean, create, publish, update } = adminConfig;
const program = new Command();
const createProgram = program.name("gitm admin").usage("<command> <type>").description(
  i18n.__("Create bugfix, release, develop and support branches")
).command("create " + createArgs(create.args));
create.options.forEach((o) => {
  createProgram.option(o.flags, o.description, o.defaultValue);
});
createProgram.action((type) => {
  const opts = ["bugfix", "release", "develop", "support"];
  const base = type === "release" ? config.master : config.release;
  const status = checkGitStatus();
  const hasBase = getIsBranchOrCommitExist(base);
  const exits = getIsBranchOrCommitExist(config[type]);
  if (!status)
    process.exit(1);
  if (!hasBase) {
    echo(
      red(
        i18n.__(
          "{{base}} branch does not exist, please create a {{base}} branch first",
          { base }
        )
      )
    );
    process.exit(1);
  }
  if (exits) {
    echo(
      red(
        i18n.__(
          "The {{branch}} branch already exists and does not need to be created again",
          { branch: config[type] }
        )
      )
    );
    process.exit(1);
  }
  if (opts.includes(type)) {
    const cmd = [
      "git fetch",
      `git checkout ${base}`,
      "git pull",
      `git checkout -b ${config[type]} ${base}`
    ];
    queue(cmd).then((data) => {
      if (data[3].status === 0) {
        echo(
          i18n.__(
            "The {{target}} branch was created successfully and is based on {{base}}, you have now switched to {{target}}\nWhen you need to publish, remember to run: {{command}}"
          ),
          {
            target: config[type],
            base,
            command: green("gitm admin publish " + config[type])
          }
        );
      }
    });
  } else {
    echo(red(i18n.__("type only allows input") + ": " + opts.join(",")));
    process.exit(1);
  }
});
const publishProgram = program.name("gitm admin").usage(
  "<command> <type> [--description [description]] [-c --combine] [--use-rebase] [-p --prod] [-b --build [app]] [--postmsg] [-f --force]"
).description(i18n.__("Release bugfix, release, support branches")).command("publish " + createArgs(publish.args));
publish.options.forEach((o) => {
  publishProgram.option(o.flags, o.description, o.defaultValue);
});
publishProgram.action(
  async (type, opt) => {
    const {
      token,
      level,
      nickname = ""
    } = userInfoApi ? await getUserToken() : {};
    const opts = ["bugfix", "release", "support"];
    const status = checkGitStatus();
    const curBranch = await getCurrentBranch();
    let isDescriptionCorrect = true;
    if (!status)
      process.exit(1);
    fetch();
    if (config.descriptionValidator) {
      const reg = getType(config.descriptionValidator) === "regexp" ? config.descriptionValidator : new RegExp(config.descriptionValidator);
      isDescriptionCorrect = opt.description && reg.test(opt.description);
    }
    const isNeedCombineBugfixToRelease = !getIsMergedTargetBranch(
      `origin/${config.bugfix}`,
      config.release,
      true
    );
    const isNeedCombineSupportToRelease = !getIsMergedTargetBranch(
      `origin/${config.support}`,
      config.release,
      true
    );
    const isNeedCombineSupportToBugfix = !getIsMergedTargetBranch(
      `origin/${config.support}`,
      config.bugfix,
      true
    );
    const isNeedCombineReleaseToMaster = !getIsMergedTargetBranch(
      `origin/${config.release}`,
      config.master,
      true
    );
    if (opts.includes(type)) {
      let cmd;
      if (!level || level < 4) {
        cmd = {
          bugfix: isNeedCombineBugfixToRelease || opt.force ? [
            "git fetch",
            `git checkout ${config.bugfix}`,
            "git pull",
            `git checkout ${config.release}`,
            "git pull",
            {
              cmd: `git merge --no-ff ${config.bugfix}`,
              config: {
                again: false,
                postmsg: opt.postmsg,
                success: i18n.__(
                  "Merge {{source}} into {{target}} successfully",
                  {
                    source: config.bugfix,
                    target: config.release
                  }
                ),
                fail: i18n.__(
                  "An error occurred merging {{source}} to {{target}}, Please follow the instructions",
                  {
                    source: config.bugfix,
                    target: config.release
                  }
                )
              }
            },
            {
              cmd: "git push",
              config: {
                again: true,
                success: i18n.__("Successful Pushed"),
                fail: i18n.__(
                  "Push failed, please follow the prompts"
                )
              }
            }
          ] : [
            {
              message: i18n.__(
                "{{source}} has been merged with {{target}}",
                {
                  source: config.bugfix,
                  target: config.release
                }
              )
            }
          ],
          support: [].concat(
            isNeedCombineSupportToRelease || opt.force ? [
              "git fetch",
              `git checkout ${config.support}`,
              "git pull",
              `git checkout ${config.release}`,
              "git pull",
              {
                cmd: `git merge --no-ff ${config.support}`,
                config: {
                  again: false,
                  success: i18n.__(
                    "Merge {{source}} into {{target}} successfully",
                    {
                      source: config.support,
                      target: config.release
                    }
                  ),
                  fail: i18n.__(
                    "An error occurred merging {{source}} to {{target}}, Please follow the instructions",
                    {
                      source: config.support,
                      target: config.release
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: i18n.__("Successful Pushed"),
                  fail: i18n.__(
                    "Push failed, please follow the prompts"
                  )
                }
              }
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.support,
                    target: config.release
                  }
                )
              }
            ]
          ).concat(
            isNeedCombineSupportToBugfix || opt.force ? [
              `git checkout ${config.bugfix}`,
              "git pull",
              {
                cmd: `git merge --no-ff ${config.support}`,
                config: {
                  again: false,
                  success: i18n.__(
                    "Merge {{source}} into {{target}} successfully",
                    {
                      source: config.support,
                      target: config.bugfix
                    }
                  ),
                  fail: i18n.__(
                    "An error occurred merging {{source}} to {{target}}, Please follow the instructions",
                    {
                      source: config.support,
                      target: config.bugfix
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: i18n.__("Successful Pushed"),
                  fail: i18n.__(
                    "Push failed, please follow the prompts"
                  )
                }
              }
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.support,
                    target: config.bugfix
                  }
                )
              }
            ]
          ),
          release: isNeedCombineReleaseToMaster || opt.force ? [
            "git fetch",
            `git checkout ${config.release}`,
            "git pull",
            `git checkout ${config.master}`,
            "git pull",
            {
              cmd: `git merge --no-ff ${config.release}`,
              config: {
                again: false,
                success: i18n.__(
                  "Merge {{source}} into {{target}} successfully",
                  {
                    source: config.release,
                    target: config.master
                  }
                ),
                fail: i18n.__(
                  "An error occurred merging {{source}} to {{target}}, Please follow the instructions",
                  {
                    source: config.release,
                    target: config.master
                  }
                )
              }
            },
            {
              cmd: "git push",
              config: {
                again: true,
                success: i18n.__("Successful Pushed"),
                fail: i18n.__(
                  "Push failed, please follow the prompts"
                )
              }
            }
          ] : [
            {
              message: i18n.__(
                "{{source}} has been merged with {{target}}",
                {
                  source: config.release,
                  target: config.master
                }
              )
            }
          ]
        };
      } else {
        if (!isDescriptionCorrect) {
          echo(
            red(
              i18n.__(
                "The description of the reason for submission does not meet the specification"
              )
            )
          );
          process.exit(1);
        }
        cmd = {
          bugfix: isNeedCombineBugfixToRelease || opt.force ? [
            {
              cmd: {
                module: mergeRequestModule,
                entry: "createMergeRequest",
                options: {
                  source_branch: config.bugfix,
                  target_branch: config.release,
                  token,
                  description: opt.description
                }
              },
              config: {
                again: true,
                success: i18n.__(
                  "Successfully created merge request"
                ),
                fail: i18n.__(
                  "There was an error creating the merge request, please follow the instructions"
                )
              }
            },
            `gitm postmsg "${i18n.__(
              "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
              {
                nickname,
                app: appName,
                source: config.bugfix,
                target: config.release
              }
            )}"`
          ] : [
            {
              message: i18n.__(
                "{{source}} has been merged with {{target}}",
                {
                  source: config.bugfix,
                  target: config.release
                }
              )
            }
          ],
          support: [].concat(
            isNeedCombineSupportToRelease || opt.force ? [
              {
                cmd: {
                  module: mergeRequestModule,
                  entry: "createMergeRequest",
                  options: {
                    source_branch: config.support,
                    target_branch: config.release,
                    token,
                    description: opt.description
                  }
                },
                config: {
                  again: true,
                  success: i18n.__(
                    "Successfully created merge request"
                  ),
                  fail: i18n.__(
                    "There was an error creating the merge request, please follow the instructions"
                  )
                }
              },
              `gitm postmsg "${i18n.__(
                "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
                {
                  nickname,
                  app: appName,
                  source: config.support,
                  target: config.release
                }
              )}"`
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.support,
                    target: config.release
                  }
                )
              }
            ]
          ).concat(
            isNeedCombineSupportToBugfix || opt.force ? [
              {
                cmd: {
                  module: mergeRequestModule,
                  entry: "createMergeRequest",
                  options: {
                    source_branch: config.support,
                    target_branch: config.bugfix,
                    token,
                    description: opt.description
                  }
                },
                config: {
                  again: true,
                  success: i18n.__(
                    "Successfully created merge request"
                  ),
                  fail: i18n.__(
                    "There was an error creating the merge request, please follow the instructions"
                  )
                }
              },
              `gitm postmsg "${i18n.__(
                "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
                {
                  nickname,
                  app: appName,
                  source: config.support,
                  target: config.bugfix
                }
              )}"`
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.support,
                    target: config.bugfix
                  }
                )
              }
            ]
          ),
          release: isNeedCombineReleaseToMaster || opt.force ? [
            {
              cmd: {
                module: mergeRequestModule,
                entry: "createMergeRequest",
                options: {
                  source_branch: config.release,
                  target_branch: config.master,
                  token,
                  description: opt.description
                }
              },
              config: {
                again: true,
                success: i18n.__(
                  "Successfully created merge request"
                ),
                fail: i18n.__(
                  "There was an error creating the merge request, please follow the instructions"
                )
              }
            },
            `gitm postmsg "${i18n.__(
              "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
              {
                nickname,
                app: appName,
                source: config.release,
                target: config.master
              }
            )}"`
          ] : [
            {
              message: i18n.__(
                "{{source}} has been merged with {{target}}",
                {
                  source: config.release,
                  target: config.master
                }
              )
            }
          ]
        };
      }
      if (type === "bugfix" && opt.prod) {
        const isNeedCombine = !getIsMergedTargetBranch(
          `origin/${config.bugfix}`,
          config.master,
          true
        );
        if (!level || level < 4) {
          cmd[type] = cmd[type].concat(
            isNeedCombine || opt.force ? [
              `git checkout ${config.master}`,
              "git pull",
              {
                cmd: `git merge --no-ff ${config.bugfix}`,
                config: {
                  again: false,
                  success: i18n.__(
                    "Merge {{source}} into {{target}} successfully",
                    {
                      source: config.bugfix,
                      target: config.master
                    }
                  ),
                  fail: i18n.__(
                    "An error occurred merging {{source}} to {{target}}, Please follow the instructions",
                    {
                      source: config.bugfix,
                      target: config.master
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: i18n.__("Successful Pushed"),
                  fail: i18n.__(
                    "Push failed, please follow the prompts"
                  )
                }
              }
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.bugfix,
                    target: config.master
                  }
                )
              }
            ]
          );
        } else {
          if (!isDescriptionCorrect) {
            echo(
              red(
                i18n.__(
                  "The description of the reason for submission does not meet the specification"
                )
              )
            );
            process.exit(1);
          }
          cmd[type] = cmd[type].concat(
            isNeedCombine || opt.force ? [
              {
                cmd: {
                  module: mergeRequestModule,
                  entry: "createMergeRequest",
                  options: {
                    source_branch: config.bugfix,
                    target_branch: config.master,
                    token,
                    description: opt.description
                  }
                },
                config: {
                  again: true,
                  success: i18n.__(
                    "Successfully created merge request"
                  ),
                  fail: i18n.__(
                    "There was an error creating the merge request, please follow the instructions"
                  )
                }
              },
              `gitm postmsg "${i18n.__(
                "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
                {
                  nickname,
                  app: appName,
                  source: config.bugfix,
                  target: config.master
                }
              )}"`
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.bugfix,
                    target: config.master
                  }
                )
              }
            ]
          );
        }
        if (opt.build && (!level || level < 4)) {
          cmd[type] = cmd[type].concat([
            {
              cmd: `gitm build ${appName} --env bug --app ${opt.build === true ? "all" : opt.build}`,
              config: {
                again: false,
                success: i18n.__(
                  "Pulling up the build was successful"
                ),
                fail: i18n.__("Failed to pull up the build")
              }
            }
          ]);
        }
      }
      if (type === "release" && opt.build && (!level || level < 4)) {
        cmd[type] = cmd[type].concat([
          {
            cmd: `gitm build ${appName} --env prod --app ${opt.build === true ? "all" : opt.build}`,
            config: {
              again: false,
              success: i18n.__(
                "Pulling up the build was successful"
              ),
              fail: i18n.__("Failed to pull up the build")
            }
          }
        ]);
      }
      if (type === "release" && opt.combine) {
        const isNeedCombine = !getIsMergedTargetBranch(
          `origin/${config.release}`,
          config.bugfix,
          true
        );
        if (opt.useRebase) {
          cmd[type] = cmd[type].concat(
            isNeedCombine || opt.force ? [
              `git checkout ${config.release}`,
              "git pull",
              `git checkout ${config.bugfix}`,
              {
                cmd: `git pull origin ${config.bugfix} --rebase`,
                config: { again: true }
              },
              {
                cmd: `git rebase ${config.release}`,
                config: {
                  again: false,
                  postmsg: opt.postmsg,
                  success: i18n.__(
                    "Merge {{source}} to {{target}} successfully",
                    {
                      source: config.release,
                      target: config.bugfix
                    }
                  ),
                  fail: i18n.__(
                    "An error occurred merging {{source} to {{target}}, please follow the instructions",
                    {
                      source: config.release,
                      target: config.bugfix
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: i18n.__("Successful Pushed"),
                  fail: i18n.__(
                    "Push failed, please follow the prompts"
                  )
                }
              }
            ] : [
              {
                message: i18n.__(
                  "{{source}} has been merged with {{target}}",
                  {
                    source: config.release,
                    target: config.bugfix
                  }
                )
              }
            ]
          );
        } else {
          if (!level || level < 4) {
            cmd[type] = cmd[type].concat(
              isNeedCombine || opt.force ? [
                `git checkout ${config.release}`,
                "git pull",
                `git checkout ${config.bugfix}`,
                "git pull",
                {
                  cmd: `git merge --no-ff ${config.release}`,
                  config: {
                    again: false,
                    postmsg: opt.postmsg,
                    success: i18n.__(
                      "Merge {{source}} into {{target}} successfully",
                      {
                        source: config.release,
                        target: config.bugfix
                      }
                    ),
                    fail: i18n.__(
                      "An error occurred merging {{source}} to {{target}}, Please follow the instructions",
                      {
                        source: config.release,
                        target: config.bugfix
                      }
                    )
                  }
                },
                {
                  cmd: "git push",
                  config: {
                    again: true,
                    success: i18n.__("Successful Pushed"),
                    fail: i18n.__(
                      "Push failed, please follow the prompts"
                    )
                  }
                }
              ] : [
                {
                  message: i18n.__(
                    "{{source}} has been merged with {{target}}",
                    {
                      source: config.release,
                      target: config.bugfix
                    }
                  )
                }
              ]
            );
          } else {
            if (!isDescriptionCorrect) {
              echo(
                red(
                  i18n.__(
                    "The description of the reason for submission does not meet the specification"
                  )
                )
              );
              process.exit(1);
            }
            cmd[type] = cmd[type].concat([
              {
                cmd: {
                  module: mergeRequestModule,
                  entry: "createMergeRequest",
                  options: {
                    source_branch: config.release,
                    target_branch: config.bugfix,
                    token,
                    description: opt.description
                  }
                },
                config: {
                  again: true,
                  success: i18n.__(
                    "Successfully created merge request"
                  ),
                  fail: i18n.__(
                    "There was an error creating the merge request, please follow the instructions"
                  )
                }
              },
              `gitm postmsg "${i18n.__(
                "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
                {
                  nickname,
                  app: appName,
                  source: config.release,
                  target: config.bugfix
                }
              )}"`
            ]);
          }
        }
      }
      let key;
      for (key in cmd) {
        cmd[key].push(`git checkout ${curBranch}`);
      }
      queue(cmd[type]);
    } else {
      echo(red(i18n.__("type only allows input") + ": " + opts.join(",")));
      process.exit(1);
    }
  }
);
const updateProgram = program.name("gitm admin").usage(
  "<command> <type> [-m --mode [mode]] [--description [description]] [--use-rebase] [--postmsg] [-f --force]"
).description(i18n.__("Update bugfix, release, support branch code")).command("update " + createArgs(update.args));
update.options.forEach((o) => {
  updateProgram.option(o.flags, o.description, o.defaultValue);
});
updateProgram.action(
  async (type, opt) => {
    const {
      token,
      level,
      nickname = ""
    } = userInfoApi ? await getUserToken() : {};
    const opts = ["bugfix", "release", "support"];
    const base = type === "release" ? config.master : config.release;
    const status = checkGitStatus();
    let mode = "", isDescriptionCorrect = true;
    if (!status)
      process.exit(1);
    if (config.descriptionValidator) {
      const reg = getType(config.descriptionValidator) === "regexp" ? config.descriptionValidator : new RegExp(config.descriptionValidator);
      isDescriptionCorrect = opt.description && reg.test(opt.description);
    }
    if (opt.mode === 1) {
      mode = " --strategy-option ours";
    } else if (opt.mode === 2) {
      mode = " --strategy-option theirs";
    }
    fetch();
    if (opts.includes(type)) {
      const isNeedCombine = !getIsMergedTargetBranch(
        `origin/${base}`,
        config[type],
        true
      );
      let cmd;
      if (isNeedCombine || opt.force) {
        if (!level || level < 4) {
          cmd = [
            "git fetch",
            `git checkout ${base}`,
            "git pull",
            `git checkout ${config[type]}`,
            {
              cmd: "git pull",
              config: { again: true }
            },
            {
              cmd: `git merge --no-ff ${base}${mode}`,
              config: {
                again: false,
                postmsg: opt.postmsg,
                success: i18n.__(
                  "Merge {{source}} to {{target}} successfully",
                  {
                    source: base,
                    target: config[type]
                  }
                ),
                fail: i18n.__(
                  "An error occurred merging {{source} to {{target}}, please follow the instructions",
                  {
                    source: base,
                    target: config[type]
                  }
                )
              }
            },
            {
              cmd: "git push",
              config: {
                again: true,
                success: i18n.__("Successful Pushed"),
                fail: i18n.__(
                  "Push failed, please follow the prompts"
                )
              }
            }
          ];
        } else {
          if (!isDescriptionCorrect) {
            echo(
              red(
                i18n.__(
                  "The description of the reason for submission does not meet the specification"
                )
              )
            );
            process.exit(1);
          }
          cmd = [
            {
              cmd: {
                module: mergeRequestModule,
                entry: "createMergeRequest",
                options: {
                  source_branch: base,
                  target_branch: config[type],
                  token,
                  description: opt.description
                }
              },
              config: {
                again: true,
                success: i18n.__(
                  "Successfully created merge request"
                ),
                fail: i18n.__(
                  "There was an error creating the merge request, please follow the instructions"
                )
              }
            },
            `gitm postmsg "${i18n.__(
              "{{nickname}} submitted a merge request for {{source}} branch to {{target}} branch in {{app}} project",
              {
                nickname,
                app: appName,
                source: base,
                target: config[type]
              }
            )}"`
          ];
        }
        if (opt.useRebase) {
          cmd = [
            "git fetch",
            `git checkout ${base}`,
            "git pull",
            `git checkout ${config[type]}`,
            {
              cmd: `git pull origin ${config[type]} --rebase`,
              config: { again: true }
            },
            {
              cmd: `git rebase ${base}`,
              config: {
                again: false,
                postmsg: opt.postmsg,
                success: i18n.__(
                  "Merge {{source}} to {{target}} successfully",
                  {
                    source: base,
                    target: config[type]
                  }
                ),
                fail: i18n.__(
                  "An error occurred merging {{source} to {{target}}, please follow the instructions",
                  {
                    source: base,
                    target: config[type]
                  }
                )
              }
            },
            {
              cmd: "git push",
              config: {
                again: true,
                success: i18n.__("Successful Pushed"),
                fail: i18n.__(
                  "Push failed, please follow the prompts"
                )
              }
            }
          ];
        }
      } else {
        cmd = [
          {
            message: i18n.__(
              "{{source}} has been merged with {{target}}",
              {
                source: base,
                target: config[type]
              }
            )
          }
        ];
      }
      queue(cmd);
    } else {
      echo(red(i18n.__("type only allows input") + ": " + opts.join(",")));
      process.exit(1);
    }
  }
);
const cleanProgram = program.name("gitm admin").usage("<command> <type>").description(i18n.__("Build cleanup job")).command("clean " + createArgs(clean.args));
clean.options.forEach((o) => {
  cleanProgram.option(o.flags, o.description, o.defaultValue);
});
cleanProgram.action((type) => {
  const opts = ["bugfix", "release", "develop", "master"];
  const status = checkGitStatus();
  if (!status)
    process.exit(1);
  if (opts.includes(type)) {
    let cmd = [
      "git fetch",
      "git checkout . -f",
      "git clean -fd",
      `git checkout ${config.master}`,
      `git branch -D ${config[type]}`,
      {
        cmd: "git remote prune origin",
        config: {
          again: true,
          success: i18n.__("Cleanup of remote branch was successful"),
          fail: i18n.__(
            "Failed to clean up remote branch, please follow the prompts"
          )
        }
      },
      "git fetch",
      `git checkout ${config[type]}`,
      "git pull"
    ];
    if (type === "master") {
      cmd = [
        "git checkout .",
        "git clean -fd",
        `git checkout ${config.master}`,
        "git clean -fd",
        "git fetch",
        "git pull"
      ];
    }
    queue(cmd);
  } else {
    echo(red(i18n.__("type only allows input") + ": " + opts.join(",")));
    process.exit(1);
  }
});
const approveProgram = program.name("gitm admin").usage("[type] [-k --key [keyword]]").description(i18n.__("Approve merge request")).command("approve " + createArgs(approve.args));
approve.options.forEach((o) => {
  approveProgram.option(o.flags, o.description, o.defaultValue);
});
approveProgram.action(() => {
  spawnSync("gitm", ["approve", "--state", "opened"], { stdio: "inherit" });
});
program.parse(process.argv);
